/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.clientserverapi.configuration;

import java.io.Serializable;
import java.net.URL;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import org.ow2.petals.clientserverapi.jbi.management.recovery.SystemRecoveryService;
import org.ow2.petals.clientserverapi.jbi.messaging.RouterService;
import org.ow2.petals.clientserverapi.system.persistence.PersistenceService;
import org.ow2.petals.clientserverapi.topology.TopologyService;
import org.ow2.petals.clientserverapi.topology.TopologyService.ContainerPropertyDefaultValues;
import org.ow2.petals.clientserverapi.topology.TopologyService.ContainerPropertyName;
import org.ow2.petals.clientserverapi.topology.TopologyService.ContainerPropertyValues;
import org.ow2.petals.clientserverapi.transport.FastRemoteTransportService;

/**
 * This class holds the overall configuration of a PEtALS container. 
 * <p>
 * It stores both the local topology information and the global server properties
 * information.<br>
 * The configuration service retrieves the configuration when it is started and
 * creates an instance of this class. The instance is exposed on the distributed
 * JNDI server.
 * </p>
 * 
 * @author Adrien Louis - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 * @since Petals 2.0
 */
public final class ContainerConfiguration implements Serializable {

    /**
     * Container state enumeration.
     */
    public static enum ContainerState {

        /**
         * Started state
         */
        STARTED,
        /**
         * Stopped state
         */
        STOPPED,
        /**
         * unknown state
         */
        UNKNOWN
    }

    /**
     * the "unique" ID for serialization
     */
    private static final long serialVersionUID = 6543213857L;

    /**
     * Default value of the timeout for processing the installation and deployment tasks.
     */
    public static final long DEFAULT_INSTALLATION_DEPLOYMENT_TASK_TIMEOUT = 120000;

    // ********** Global ************//
    /**
     * The location of the local configuration
     */
    private URL serverPropertiesUrl;

    /**
     * Subdomain name.
     */
    private String subdomainName;

    /**
     * Container name.
     */
    private String name;

    /**
     * Container description
     */
    private String description;

    /**
     * Host of the container.
     */
    private String host;

    /**
     * State of the container.
     */
    private ContainerState state = ContainerState.UNKNOWN;

    /**
     * Data root path
     */
    private String dataRootPath;

    /**
     * Repository directory path.
     */
    private String repositoryDirectoryPath;

    /**
     * The work directory where temporary things are stored
     */
    private String workDirectoryPath;

    /**
     * Exchange validation.<br>
     * If <code>true</code>, used by the Address resolver to know if a component
     * is agree to receive a exchange from a consumer.
     */
    private boolean exchangeValidation;

    /**
     * The timeout for processing the installation and deployment tasks.
     */
    private long taskTimeout = DEFAULT_INSTALLATION_DEPLOYMENT_TASK_TIMEOUT;

    /**
     * Flag indicating if the JBI classLoaders must be isolated from the
     * container ClassLoader.
     */
    private boolean isolateJBIClassLoaders;

    // ************ Admin **************//

    /**
     * Login to connect to the node.
     */
    private String user;

    /**
     * Password to connect to the node.
     */
    private String password;

    /**
     * Port of the RMI adaptor for JMX.
     */
    private int jmxRMIConnectorPort;

    // ***** Router *****//

    /**
     * Router Quality of Service. {@see RouterService}
     */
    private String routerQOS = RouterService.DEFAULT_POLICY;

    /**
     * Router Strategy. {@see EndpointResolverModule}
     */
    private String routerStrategy = RouterService.DEFAULT_PLATFORM_STRATEGY;

    /**
     * Number of attempt to send a message
     */
    private short routerSendAttempt = RouterService.DEFAULT_SEND_ATTEMPTS;

    /**
     * Delay between each send attempt
     */
    private int routerSendDelay = RouterService.DEFAULT_SEND_DELAY;

    // ***** Topology *****//

    /**
     * The topology pass-phrase
     */
    private String topologyPassPhrase = null;

    /**
     * Max wait time to get a lock on the dynamic topology in the registry
     */
    private int topologyDynamicLockWaitTime = TopologyService.DEFAULT_DYNAMIC_TOPOLOGY_LOCK_WAIT_TIME;

    /**
     * Waiting time before to start the timer task associated to the topology pinger
     */
    private long topologyPingerStartDelay = TopologyService.DEFAULT_TOPOLOGY_PINGER_START_DELAY;

    /**
     * Delay between two runs of the topology pinger
     */
    private long topologyPingerPeriodDelay = TopologyService.DEFAULT_TOPOLOGY_PINGER_PERIOD_DELAY;

    // ***** Transporter *****//

    /**
     * Port of the TCP communication.
     */
    private int tcpPort = 0;

    /**
     * The SSL key password
     */
    private String sslKeyPassword;

    /**
     * Max size of the transporter queues. Default value: 10000
     */
    private int transporterQueueMaxSize = RouterService.DEFAULT_TRANSPORT_QUEUE_MAX_SIZE;

    /**
     * Offering timeout of component transporter queues, in milliseconds.
     * Default value : 2500ms
     */
    private long transporterQueueOfferingTimeout = RouterService.DEFAULT_TRANSPORT_QUEUE_OFFERING_TIMEOUT;

    /**
     * Number of message that can be received via TCP at the same time
     */
    private int tcpReceivers = 10;

    /**
     * Number of message that can be send via TCP at the same time, per
     * component
     */
    private int tcpSenders = FastRemoteTransportService.DEFAULT_SENDERS;

    /**
     * TCP Connection timeout
     */
    private long tcpConnectionTimeout = FastRemoteTransportService.DEFAULT_CONNECTION_TIMEOUT;

    /**
     * TCP send timeout
     */
    private long tcpSendTimeout = FastRemoteTransportService.DEFAULT_SEND_TIMEOUT;

    /**
     * The delay before running the 'sender' eviction thread
     */
    private long tcpSenderEvictorDelay = FastRemoteTransportService.DEFAULT_SEND_EVICTOR_DELAY;

    /**
     * The delay before an idle 'sender' is set evictable
     */
    private long tcpSenderEvictableDelay = FastRemoteTransportService.DEFAULT_SEND_EVICTABLE_DELAY;

    // ***** Security *****//

    /**
     * The SSL Keystore file
     */
    private String sslKeystore;

    /**
     * The SSL Keystore password
     */
    private String sslKeystorePassword;

    /**
     * The SSL Truststore file
     */
    private String sslTruststore;

    /**
     * The SSL Truststore password
     */
    private String sslTruststorePassword;

    // ***** Persistence *****//

    /**
     * The persistence database driver.
     */
    private String persistenceDatabaseDriver = PersistenceService.HSQLDB_DRIVER;

    /**
     * The persistence database location.
     */
    private String persistenceDatabaseUrl;

    /**
     * The persistence database user.
     */
    private String persistenceDatabaseUser = PersistenceService.DEFAULT_USER;

    /**
     * The persistence database password.
     */
    private String persistenceDatabasePassword = PersistenceService.DEFAULT_PASSWORD;

    // ***** System recovery *****//
    
    /**
     * The core size of the system recovery thread pool
     */
    private int recoveryCorePoolSize = SystemRecoveryService.DEFAULT_THREADPOOL_CORESIZE;
    
    /**
     * The keep alive time of the system recovery thread pool, in second
     */
    private long recoveryKeepAliveTime = SystemRecoveryService.DEFAULT_THREADPOOL_KEEPALIVETIME;

    // ***** Persistence *****//

    /**
     * The persistence duration.
     */
    private long persistenceDuration = PersistenceService.DEFAULT_DURATION;

    /**
     * The persistence SQL request fetch size.
     */
    private int persistenceFetchSize = PersistenceService.DEFAULT_FETCHSIZE;

    /**
     * The additional configuration in the server local configuration (ie. the
     * content of '<code>server.properties</code>' or equivalent) defined by the
     * user. Get all the '<code>user.*</code>' key/value pairs.
     */
    private Map<String, String> userConfiguration;

    /**
     * The additional configuration in the server local configuration (ie. the
     * content of '<code>server.properties</code>' or equivalent) that is
     * present but a priori not used.
     */
    private Map<String, String> extraConfiguration;

    /**
     * @return the persistenceFetchSize
     */
    public int getPersistenceFetchSize() {
        return persistenceFetchSize;
    }

    /**
     * @param persistenceFetchSize
     *            the persistenceFetchSize to set
     */
    public void setPersistenceFetchSize(int persistenceFetchSize) {
        this.persistenceFetchSize = persistenceFetchSize;
    }

    /**
     * Default constructor
     */
    public ContainerConfiguration() {
        super();
    }

    /**
     * Creates a {@link ContainerConfiguration} from a {@link Map}
     */
    public ContainerConfiguration(final Map<String, String> containerConfigurationMap) {
        this.name = containerConfigurationMap.get(ContainerPropertyName.CONF_CONTAINER_NAME);
        this.description = containerConfigurationMap.get(ContainerPropertyName.CONF_CONTAINER_DESCRIPTION);
        this.subdomainName = containerConfigurationMap.get(ContainerPropertyName.CONF_SUBDOMAIN_NAME);
        this.host = containerConfigurationMap.get(ContainerPropertyName.CONF_HOST);
        this.user = containerConfigurationMap.get(ContainerPropertyName.CONF_USER);
        this.password = containerConfigurationMap.get(ContainerPropertyName.CONF_PWD);

        final String jmxRMIConnectorPort = containerConfigurationMap.get(ContainerPropertyName.CONF_JMX_RMI_PORT);
        this.jmxRMIConnectorPort = ContainerPropertyDefaultValues.CONF_JMX_RMI_PORT.equals(jmxRMIConnectorPort) ? 0
                : Integer.parseInt(jmxRMIConnectorPort);

        final String tcpPort = containerConfigurationMap.get(ContainerPropertyName.CONF_TRANSPORT_TCP_PORT);
        this.tcpPort = ContainerPropertyDefaultValues.CONF_TRANSPORT_TCP_PORT.equals(tcpPort) ? 0 : Integer
                .parseInt(tcpPort);

        final String stateStr = containerConfigurationMap.get(ContainerPropertyName.CONF_STATE);
        if (ContainerPropertyValues.STATE_STARTED.equals(stateStr)) {
            this.state = ContainerState.STARTED;
        } else if (ContainerPropertyValues.STATE_STOPPED.equals(stateStr)) {
            this.state = ContainerState.STOPPED;
        } else {
            this.state = ContainerState.UNKNOWN;
        }
    }

    /**
     * @return the serialVersionUID
     */
    public static long getSerialVersionUID() {
        return serialVersionUID;
    }

    /**
     * @return the description
     */
    public String getDescription() {
        return this.description;
    }

    /**
     * @return the host
     */
    public String getHost() {
        return this.host;
    }

    /**
     * @return the jmxRMIConnectorPort
     */
    public int getJmxRMIConnectorPort() {
        return this.jmxRMIConnectorPort;
    }

    /**
     * @return the name
     */
    public String getName() {
        return this.name;
    }

    /**
     * @return the password
     */
    public String getPassword() {
        return this.password;
    }

    /**
     * 
     * @return
     */
    public long getPersistenceDuration() {
        return persistenceDuration;
    }

    /**
     * @return the persistenceDatabaseDriver
     */
    public String getPersistenceDatabaseDriver() {
        return this.persistenceDatabaseDriver;
    }

    /**
     * @return the persistenceDatabasePassword
     */
    public String getPersistenceDatabasePassword() {
        return this.persistenceDatabasePassword;
    }

    /**
     * @return the persistenceDatabaseUrl
     */
    public String getPersistenceDatabaseUrl() {
        return this.persistenceDatabaseUrl;
    }

    /**
     * @return the persistenceDatabaseUser
     */
    public String getPersistenceDatabaseUser() {
        return this.persistenceDatabaseUser;
    }

    /**
     * @return the repositoryDirectoryPath
     */
    public String getRepositoryDirectoryPath() {
        return this.repositoryDirectoryPath;
    }

    /**
     * @return the routerQOS
     */
    public String getRouterQOS() {
        return this.routerQOS;
    }

    /**
     * @return the routerSendAttempt
     */
    public short getRouterSendAttempt() {
        return this.routerSendAttempt;
    }

    /**
     * @return the routerSendDelay
     */
    public int getRouterSendDelay() {
        return this.routerSendDelay;
    }

    /**
     * @return the routerStrategy
     */
    public String getRouterStrategy() {
        return this.routerStrategy;
    }

    /**
     * @return the sslKeyPassword
     */
    public String getSSLKeyPassword() {
        return this.sslKeyPassword;
    }

    /**
     * @return the sslKeystore
     */
    public String getSSLKeystore() {
        return this.sslKeystore;
    }

    /**
     * @return the sslKeystorePassword
     */
    public String getSSLKeystorePassword() {
        return this.sslKeystorePassword;
    }

    /**
     * @return the sslTruststore
     */
    public String getSSLTruststore() {
        return this.sslTruststore;
    }

    /**
     * @return the sslTruststorePassword
     */
    public String getSSLTruststorePassword() {
        return this.sslTruststorePassword;
    }

    /**
     * @return the state
     */
    public ContainerState getState() {
        return this.state;
    }

    /**
     * @return the subdomainName
     */
    public String getSubdomainName() {
        return this.subdomainName;
    }

    /**
     * @return the timeout for processing the installation and deployment tasks.
     */
    public long getTaskTimeout() {
        return this.taskTimeout;
    }

    /**
     * @return the tcpConnectionTimeout
     */
    public long getTCPConnectionTimeout() {
        return this.tcpConnectionTimeout;
    }

    /**
     * @return the tcpPort
     */
    public int getTCPPort() {
        return this.tcpPort;
    }

    /**
     * @return the tcpReceivers
     */
    public int getTCPReceivers() {
        return this.tcpReceivers;
    }

    /**
     * @return the tcpSenderEvictableDelay
     */
    public long getTCPSenderEvictableDelay() {
        return this.tcpSenderEvictableDelay;
    }

    /**
     * @return the tcpSenderEvictorDelay
     */
    public long getTCPSenderEvictorDelay() {
        return this.tcpSenderEvictorDelay;
    }

    /**
     * @return the tcpSenders
     */
    public int getTCPSenders() {
        return this.tcpSenders;
    }

    /**
     * @return the tcpSendTimeout
     */
    public long getTCPSendTimeout() {
        return this.tcpSendTimeout;
    }

    public String getTopologyPassPhrase() {
        return this.topologyPassPhrase;
    }

    /**
     * @return The dynamic topology lock wait time, in seconds
     */
    public int getTopologyDynamicLockWaitTime() {
        return this.topologyDynamicLockWaitTime;
    }

    /**
     * @return The waiting time before to start the timer task associated to the topology pinger.
     */
    public long getTopologyPingerStartDelay() {
        return this.topologyPingerStartDelay;
    }

    /**
     * @return The delay between two runs of the topology pinger.
     */
    public long getTopologyPingerPeriodDelay() {
        return this.topologyPingerPeriodDelay;
    }

    /**
     * @return the user
     */
    public String getUser() {
        return this.user;
    }

    /**
     * @return the userConfiguration
     */
    public Map<String, String> getUserConfiguration() {
        return this.userConfiguration;
    }

    /**
     * @return The extra properties. A properties is an extra property if not
     *         directly used by the micro-kernel but by an extension or by an
     *         dedicated Fractal component implementation. The {@link Map}
     *         returned is never <code>null</code>, can be empty.
     */
    public Map<String, String> getExtraConfiguration() {
        return this.extraConfiguration;
    }

    /**
     * @return the workDirectoryPath
     */
    public String getWorkDirectoryPath() {
        return this.workDirectoryPath;
    }

    /**
     * @return the exchangeValidation
     */
    public boolean isExchangeValidation() {
        return this.exchangeValidation;
    }

    /**
     * @return the isolateJBIClassLoaders
     */
    public boolean isIsolateJBIClassLoaders() {
        return this.isolateJBIClassLoaders;
    }

    @Override
    public String toString() {
        StringBuilder toStringBuilder = new StringBuilder();
        toStringBuilder.append("Container ").append(this.getName()).append(" on ")
                .append(this.getHost()).append(" in sub-domain ").append(this.getSubdomainName());
        if (this.getDescription() != null) {
            toStringBuilder.append("\n\t\t\tDescription :\n\t\t\t  ").append(this.getDescription());
        }
        toStringBuilder.append("\n\t\t\t  - RepositoryDirectory:").append(
                this.getRepositoryDirectoryPath());
        toStringBuilder.append("\n\t\t\t  - WorkDirectory:").append(
                this.getWorkDirectoryPath());
        toStringBuilder.append("\n\t\t\t  - ExchangeValidation:").append(
                this.isExchangeValidation());
        toStringBuilder.append("\n\t\t\t  - RouterQOS:").append(this.getRouterQOS());
        toStringBuilder.append("\n\t\t\t  - RoutingStrategy:").append(this.getRouterStrategy());
        toStringBuilder.append("\n\t\t\t  - TaskTimeout:").append(this.getTaskTimeout());
        toStringBuilder.append("\n\t\t\t  - IsolateClassLoaders:").append(
                this.isIsolateJBIClassLoaders());
        toStringBuilder.append("\n\t\t\t  - User:").append(
                (this.getUser() != null) ? this.getUser() : "none");
        if (this.getPassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getPassword().length());
            while (hiddenPassword.length() < this.getPassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - Password:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - Password:none");
        }
        toStringBuilder.append("\n\t\t\t  - JMXRMIConnectionPort:").append(
                this.getJmxRMIConnectorPort());
        toStringBuilder.append("\n\t\t\t  - TCPPort:").append(
                (this.getTCPPort() != 0) ? this.getTCPPort() : "none");
        if (this.getSSLKeyPassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getSSLKeyPassword().length());
            while (hiddenPassword.length() < this.getSSLKeyPassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - sslKeyPassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - sslKeyPassword:none");
        }
        toStringBuilder.append("\n\t\t\t  - Keystore:").append(
                (this.getSSLKeystore() != null) ? this.getSSLKeystore() : "none");
        if (this.getSSLKeystorePassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getSSLKeystorePassword().length());
            while (hiddenPassword.length() < this.getSSLKeystorePassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - KeystorePassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - KeystorePassword:none");
        }
        toStringBuilder.append("\n\t\t\t  - Truststore:").append(
                (this.getSSLTruststore() != null) ? this.getSSLTruststore() : "none");
        if (this.getSSLTruststorePassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getSSLTruststorePassword().length());
            while (hiddenPassword.length() < this.getSSLTruststorePassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - TruststorePassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - TruststorePassword:none");
        }
        if (this.getTCPPort() != 0) {
            toStringBuilder.append("\n\t\t\t  - TCPReceivers:").append(this.getTCPReceivers());
            toStringBuilder.append("\n\t\t\t  - TCPSenders:").append(this.getTCPSenders());
            toStringBuilder.append("\n\t\t\t  - TCPConnectionTimeout:").append(
                    this.getTCPConnectionTimeout());
            toStringBuilder.append("\n\t\t\t  - TCPSendTimeout:").append(this.getTCPSendTimeout());
            toStringBuilder.append("\n\t\t\t  - TCPSenderEvictorDelay:").append(
                    this.getTCPSenderEvictorDelay());
            toStringBuilder.append("\n\t\t\t  - TCPSenderEvictableDelay:").append(
                    this.getTCPSenderEvictableDelay());
        }
        toStringBuilder.append("\n\t\t\t  - TopologyDynamicLockWaitTime:")
                .append(this.getTopologyDynamicLockWaitTime());
        toStringBuilder.append("\n\t\t\t  - TopologyPingerStartDelay:").append(this.getTopologyPingerStartDelay());
        toStringBuilder.append("\n\t\t\t  - TopologyPingerPeriodDelay:").append(this.getTopologyPingerPeriodDelay());
        toStringBuilder.append("\n\t\t\t  - State:").append(this.getState().name());
        toStringBuilder.append("\n\t\t\t  - PersistenceDatabaseDriver:").append(this.getPersistenceDatabaseDriver());
        toStringBuilder.append("\n\t\t\t  - PersistenceDatabaseUrl:").append(this.getPersistenceDatabaseUrl());
        toStringBuilder.append("\n\t\t\t  - PersistenceDatabaseUser:").append(this.getPersistenceDatabaseUser());
        if (this.getPersistenceDatabasePassword() != null) {
            final StringBuffer hiddenPassword = new StringBuffer(this.getPersistenceDatabasePassword()
                    .length());
            while (hiddenPassword.length() < this.getPersistenceDatabasePassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - PersistenceDatabasePassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - PersistenceDatabasePassword:none");
        }
        toStringBuilder.append("\n\t\t\t  - PersistenceDuration:").append(
                this.getPersistenceDuration());

        if (this.getUserConfiguration() != null) {
            StringBuffer userBuffer = new StringBuffer();
            Iterator<String> iter = this.getUserConfiguration().keySet().iterator();
            while (iter.hasNext()) {
                String key = iter.next();
                userBuffer.append("\n\t\t\t\t").append(key).append('=')
                        .append(this.getUserConfiguration().get(key));
            }
            toStringBuilder.append("\n\t\t\t  - User properties:").append(userBuffer.toString());
        }

        return toStringBuilder.toString();
    }

    /**
     * @param description
     *            the description to set
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * @param exchangeValidation
     *            the exchangeValidation to set
     */
    public void setExchangeValidation(boolean exchangeValidation) {
        this.exchangeValidation = exchangeValidation;
    }

    /**
     * @param host
     *            the host to set
     */
    public void setHost(String host) {
        this.host = host;
    }

    /**
     * @param isolateJBIClassLoaders
     *            the isolateJBIClassLoaders to set
     */
    public void setIsolateJBIClassLoaders(boolean isolateJBIClassLoaders) {
        this.isolateJBIClassLoaders = isolateJBIClassLoaders;
    }

    /**
     * @param jmxRMIConnectorPort
     *            the jmxRMIConnectorPort to set
     */
    public void setJmxRMIConnectorPort(int jmxRMIConnectorPort) {
        this.jmxRMIConnectorPort = jmxRMIConnectorPort;
    }

    /**
     * @param name
     *            the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @param password
     *            the password to set
     */
    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * 
     * @param persistenceDuration
     */
    public void setPersistenceDuration(long persistenceDuration) {
        this.persistenceDuration = persistenceDuration;
    }

    /**
     * @param persistenceDatabaseDriver
     *            the persistenceDatabaseDriver to set
     */
    public void setPersistenceDatabaseDriver(final String persistenceDatabaseDriver) {
        this.persistenceDatabaseDriver = persistenceDatabaseDriver;
    }

    /**
     * @param persistenceDatabasePassword
     *            the persistenceDatabasePassword to set
     */
    public void setPersistenceDatabasePassword(final String persistenceDatabasePassword) {
        this.persistenceDatabasePassword = persistenceDatabasePassword;
    }

    /**
     * @param persistenceDatabaseUrl
     *            the persistenceDatabaseUrl to set
     */
    public void setPersistenceDatabaseUrl(final String persistenceDatabaseUrl) {
        this.persistenceDatabaseUrl = persistenceDatabaseUrl;
    }

    /**
     * @param persistenceDatabaseUser
     *            the persistenceDatabaseUser to set
     */
    public void setPersistenceDatabaseUser(final String persistenceDatabaseUser) {
        this.persistenceDatabaseUser = persistenceDatabaseUser;
    }

    /**
     * @param repositoryDirectoryPath
     *            the repositoryDirectoryPath to set
     */
    public void setRepositoryDirectoryPath(String repositoryDirectoryPath) {
        this.repositoryDirectoryPath = repositoryDirectoryPath;
    }

    /**
     * @param routerQOS
     *            the routerQOS to set
     */
    public void setRouterQOS(String routerQOS) {
        this.routerQOS = routerQOS;
    }

    /**
     * @param routerSendAttempt
     *            the routerSendAttempt to set
     */
    public void setRouterSendAttempt(short routerSendAttempt) {
        this.routerSendAttempt = routerSendAttempt;
    }

    /**
     * @param routerSendDelay
     *            the routerSendDelay to set
     */
    public void setRouterSendDelay(int routerSendDelay) {
        this.routerSendDelay = routerSendDelay;
    }

    /**
     * @param routerStrategy
     *            the routerStrategy to set
     */
    public void setRouterStrategy(String routerStrategy) {
        this.routerStrategy = routerStrategy;
    }

    /**
     * @param sslKeyPassword
     *            the sslKeyPassword to set
     */
    public void setSSLKeyPassword(String sslKeyPassword) {
        this.sslKeyPassword = sslKeyPassword;
    }

    /**
     * @param sslKeystore
     *            the sslKeystore to set
     */
    public void setSSLKeystore(String sslKeystore) {
        this.sslKeystore = sslKeystore;
    }

    /**
     * @param sslKeystorePassword
     *            the sslKeystorePassword to set
     */
    public void setSSLKeystorePassword(String sslKeystorePassword) {
        this.sslKeystorePassword = sslKeystorePassword;
    }

    /**
     * @param sslTruststore
     *            the sslTruststore to set
     */
    public void setSSLTruststore(String sslTruststore) {
        this.sslTruststore = sslTruststore;
    }

    /**
     * @param sslTruststorePassword
     *            the sslTruststorePassword to set
     */
    public void setSSLTruststorePassword(String sslTruststorePassword) {
        this.sslTruststorePassword = sslTruststorePassword;
    }

    /**
     * @param state
     *            the state to set
     */
    public void setState(ContainerState state) {
        this.state = state;
    }

    /**
     * @param subdomainName
     *            the subdomainName to set
     */
    public void setSubdomainName(String subdomainName) {
        this.subdomainName = subdomainName;
    }

    /**
     * @param taskTimeout
     *            the timeout for processing the installation and deployment tasks.
     */
    public void setTaskTimeout(final long taskTimeout) {
        this.taskTimeout = taskTimeout;
    }

    /**
     * @param tcpConnectionTimeout
     *            the tcpConnectionTimeout to set
     */
    public void setTCPConnectionTimeout(long tcpConnectionTimeout) {
        this.tcpConnectionTimeout = tcpConnectionTimeout;
    }

    /**
     * @param tcpPort
     *            the tcpPort to set
     */
    public void setTCPPort(int tcpPort) {
        this.tcpPort = tcpPort;
    }

    /**
     * @param tcpReceivers
     *            the tcpReceivers to set
     */
    public void setTCPReceivers(int tcpReceivers) {
        this.tcpReceivers = tcpReceivers;
    }

    /**
     * @param tcpSenderEvictableDelay
     *            the tcpSenderEvictableDelay to set
     */
    public void setTCPSenderEvictableDelay(long tcpSenderEvictableDelay) {
        this.tcpSenderEvictableDelay = tcpSenderEvictableDelay;
    }

    /**
     * @param tcpSenderEvictorDelay
     *            the tcpSenderEvictorDelay to set
     */
    public void setTCPSenderEvictorDelay(long tcpSenderEvictorDelay) {
        this.tcpSenderEvictorDelay = tcpSenderEvictorDelay;
    }

    /**
     * @param tcpSenders
     *            the tcpSenders to set
     */
    public void setTCPSenders(int tcpSenders) {
        this.tcpSenders = tcpSenders;
    }

    /**
     * @param tcpSendTimeout
     *            the tcpSendTimeout to set
     */
    public void setTCPSendTimeout(long tcpSendTimeout) {
        this.tcpSendTimeout = tcpSendTimeout;
    }

    /**
     * 
     * @param topologyPassPhrase
     */
    public void setTopologyPassPhrase(final String topologyPassPhrase) {
        this.topologyPassPhrase = topologyPassPhrase;
    }

    /**
     * Set the dynamic topology lock wait time
     * 
     * @param topologyDynamicLockWaitTime
     *            The dynamic topology lock wait time to set
     */
    public void setTopologyDynamicLockWaitTime(final int topologyDynamicLockWaitTime) {
        this.topologyDynamicLockWaitTime = topologyDynamicLockWaitTime;
    }

    /**
     * Set the waiting time before to start the timer task associated to the topology pinger
     * 
     * @param topologyPingerStartDelay
     *            The wait time to set
     */
    public void setTopologyPingerStartDelay(final long topologyPingerStartDelay) {
        this.topologyPingerStartDelay = topologyPingerStartDelay;
    }

    /**
     * Set the delay between two runs of the topology pinger
     * 
     * @param topologyPingerPeriodDelay
     *            The delay between two runs
     */
    public void setTopologyPingerPeriodDelay(final long topologyPingerPeriodDelay) {
        this.topologyPingerPeriodDelay = topologyPingerPeriodDelay;
    }

    /**
     * @param user
     *            the user to set
     */
    public void setUser(String user) {
        this.user = user;
    }

    /**
     * @param userConfiguration
     *            the userConfiguration to set
     */
    public void setUserConfiguration(Map<String, String> userConfiguration) {
        this.userConfiguration = userConfiguration;
    }

    /**
     * @param extraConfiguration
     *            the extraConfiguration to set
     */
    public void setExtraConfiguration(Map<String, String> extraConfiguration) {
        this.extraConfiguration = extraConfiguration;
    }

    /**
     * @param workDirectoryPath
     *            the workDirectoryPath to set
     */
    public void setWorkDirectoryPath(String workDirectoryPath) {
        this.workDirectoryPath = workDirectoryPath;
    }

    public int getRecoveryCorePoolSize() {
        return recoveryCorePoolSize;
    }

    public void setRecoveryCorePoolSize(int recoveryCorePoolSize) {
        this.recoveryCorePoolSize = recoveryCorePoolSize;
    }

    public long getRecoveryKeepAliveTime() {
        return recoveryKeepAliveTime;
    }

    public void setRecoveryKeepAliveTime(long recoveryKeepAliveTime) {
        this.recoveryKeepAliveTime = recoveryKeepAliveTime;
    }

    /**
     * @return the transporterQueueMaxSize
     */
    public int getTransporterQueueMaxSize() {
        return transporterQueueMaxSize;
    }

    /**
     * @param transporterQueueMaxSize
     *            the transporterQueueMaxSize to set
     */
    public void setTransporterQueueMaxSize(int transporterQueueMaxSize) {
        this.transporterQueueMaxSize = transporterQueueMaxSize;
    }

    /**
     * @return the transporterQueueOfferingTimeout
     */
    public long getTransporterQueueOfferingTimeout() {
        return transporterQueueOfferingTimeout;
    }

    /**
     * @param transporterQueueOfferingTimeout
     *            the transporterQueueOfferingTimeout to set
     */
    public void setTransporterQueueOfferingTimeout(long transporterQueueOfferingTimeout) {
        this.transporterQueueOfferingTimeout = transporterQueueOfferingTimeout;
    }

    /**
     * @return the serverPropertiesUrl
     */
    public URL getServerPropertiesUrl() {
        return this.serverPropertiesUrl;
    }

    /**
     * @param serverPropertiesUrl
     *            the serverPropertiesUrl to set
     */
    public void setServerPropertiesUrl(final URL serverPropertiesUrl) {
        this.serverPropertiesUrl = serverPropertiesUrl;
    }

    /**
     * @return the dataRootPath
     */
    public String getDataRootPath() {
        return this.dataRootPath;
    }

    /**
     * @param dataRootPath
     *            the dataRootPath to set
     */
    public void setDataRootPath(final String dataRootPath) {
        this.dataRootPath = dataRootPath;
    }

    /**
     * Convert the container configuration into a key/value map form.
     * 
     * @param isSecurityOk
     *            if <code>true</code>, sensible information are returned. Otherwise, sensible information are skipped.
     * @return The container configuration as key/value map form.
     */
    public Map<String, String> toMap(final boolean isSecurityOk) {

        final Map<String, String> containerMap = new Hashtable<String, String>();
        containerMap.put(ContainerPropertyName.CONF_CONTAINER_NAME, this.getName());
        if (this.getDescription() != null) {
            containerMap.put(ContainerPropertyName.CONF_CONTAINER_DESCRIPTION, this.getDescription());
        }
        containerMap.put(ContainerPropertyName.CONF_SUBDOMAIN_NAME, this.getSubdomainName());
        containerMap.put(ContainerPropertyName.CONF_HOST, this.getHost());

        if (isSecurityOk) {
            containerMap.put(ContainerPropertyName.CONF_USER, this.getUser());
            containerMap.put(ContainerPropertyName.CONF_PWD, this.getPassword());
        }

        if (this.getJmxRMIConnectorPort() != 0) {
            containerMap.put(ContainerPropertyName.CONF_JMX_RMI_PORT, Integer.toString(this.getJmxRMIConnectorPort()));
        } else {
            containerMap.put(ContainerPropertyName.CONF_JMX_RMI_PORT, ContainerPropertyDefaultValues.CONF_JMX_RMI_PORT);
        }

        if (this.getTCPPort() != 0) {
            containerMap.put(ContainerPropertyName.CONF_TRANSPORT_TCP_PORT, Integer.toString(this.getTCPPort()));
        } else {
            containerMap.put(ContainerPropertyName.CONF_TRANSPORT_TCP_PORT,
                    ContainerPropertyDefaultValues.CONF_TRANSPORT_TCP_PORT);
        }

        if (ContainerConfiguration.ContainerState.STARTED.equals(this.getState())) {
            containerMap.put(ContainerPropertyName.CONF_STATE, ContainerPropertyValues.STATE_STARTED);
        } else {
            containerMap.put(ContainerPropertyName.CONF_STATE, ContainerPropertyValues.STATE_STOPPED);
        }
        return containerMap;
    }

    /* Generated by Eclipse */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((dataRootPath == null) ? 0 : dataRootPath.hashCode());
        result = prime * result + ((description == null) ? 0 : description.hashCode());
        result = prime * result + (exchangeValidation ? 1231 : 1237);
        result = prime * result
                + ((extraConfiguration == null) ? 0 : extraConfiguration.hashCode());
        result = prime * result + ((host == null) ? 0 : host.hashCode());
        result = prime * result + (isolateJBIClassLoaders ? 1231 : 1237);
        result = prime * result + jmxRMIConnectorPort;
        result = prime * result + ((name == null) ? 0 : name.hashCode());
        result = prime * result + ((password == null) ? 0 : password.hashCode());
        result = prime * result + (int) (persistenceDuration ^ (persistenceDuration >>> 32));
        result = prime * result + persistenceFetchSize;
        result = prime * result + recoveryCorePoolSize;
        result = prime * result + (int) (recoveryKeepAliveTime ^ (recoveryKeepAliveTime >>> 32));
        result = prime * result + ((persistenceDatabaseDriver == null) ? 0 : persistenceDatabaseDriver.hashCode());
        result = prime * result + ((persistenceDatabasePassword == null) ? 0 : persistenceDatabasePassword.hashCode());
        result = prime * result + ((persistenceDatabaseUrl == null) ? 0 : persistenceDatabaseUrl.hashCode());
        result = prime * result + ((persistenceDatabaseUser == null) ? 0 : persistenceDatabaseUser.hashCode());
        result = prime * result
                + ((repositoryDirectoryPath == null) ? 0 : repositoryDirectoryPath.hashCode());
        result = prime * result + ((routerQOS == null) ? 0 : routerQOS.hashCode());
        result = prime * result + routerSendAttempt;
        result = prime * result + routerSendDelay;
        result = prime * result + ((routerStrategy == null) ? 0 : routerStrategy.hashCode());
        result = prime * result
                + ((serverPropertiesUrl == null) ? 0 : serverPropertiesUrl.hashCode());
        result = prime * result + ((sslKeyPassword == null) ? 0 : sslKeyPassword.hashCode());
        result = prime * result + ((sslKeystore == null) ? 0 : sslKeystore.hashCode());
        result = prime * result
                + ((sslKeystorePassword == null) ? 0 : sslKeystorePassword.hashCode());
        result = prime * result + ((sslTruststore == null) ? 0 : sslTruststore.hashCode());
        result = prime * result
                + ((sslTruststorePassword == null) ? 0 : sslTruststorePassword.hashCode());
        result = prime * result + ((state == null) ? 0 : state.hashCode());
        result = prime * result + ((subdomainName == null) ? 0 : subdomainName.hashCode());
        result = prime * result + (int) (taskTimeout ^ (taskTimeout >>> 32));
        result = prime * result + (int) (tcpConnectionTimeout ^ (tcpConnectionTimeout >>> 32));
        result = prime * result + tcpPort;
        result = prime * result + tcpReceivers;
        result = prime * result + (int) (tcpSendTimeout ^ (tcpSendTimeout >>> 32));
        result = prime * result
                + (int) (tcpSenderEvictableDelay ^ (tcpSenderEvictableDelay >>> 32));
        result = prime * result + (int) (tcpSenderEvictorDelay ^ (tcpSenderEvictorDelay >>> 32));
        result = prime * result + tcpSenders;
        result = prime * result
                + ((topologyPassPhrase == null) ? 0 : topologyPassPhrase.hashCode());
        result = prime * result + (int) (topologyDynamicLockWaitTime ^ (topologyDynamicLockWaitTime >>> 32));
        result = prime * result + (int) (topologyPingerStartDelay ^ (topologyPingerStartDelay >>> 32));
        result = prime * result + (int) (topologyPingerPeriodDelay ^ (topologyPingerPeriodDelay >>> 32));
        result = prime * result + transporterQueueMaxSize;
        result = prime
                * result
                + (int) (transporterQueueOfferingTimeout ^ (transporterQueueOfferingTimeout >>> 32));
        result = prime * result + ((user == null) ? 0 : user.hashCode());
        result = prime * result + ((userConfiguration == null) ? 0 : userConfiguration.hashCode());
        result = prime * result + ((workDirectoryPath == null) ? 0 : workDirectoryPath.hashCode());
        return result;
    }

    /* Generated by Eclipse */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (!(obj instanceof ContainerConfiguration)) {
            return false;
        }
        ContainerConfiguration other = (ContainerConfiguration) obj;
        if (dataRootPath == null) {
            if (other.dataRootPath != null) {
                return false;
            }
        } else if (!dataRootPath.equals(other.dataRootPath)) {
            return false;
        }
        if (description == null) {
            if (other.description != null) {
                return false;
            }
        } else if (!description.equals(other.description)) {
            return false;
        }
        if (exchangeValidation != other.exchangeValidation) {
            return false;
        }
        if (extraConfiguration == null) {
            if (other.extraConfiguration != null) {
                return false;
            }
        } else if (!extraConfiguration.equals(other.extraConfiguration)) {
            return false;
        }
        if (host == null) {
            if (other.host != null) {
                return false;
            }
        } else if (!host.equals(other.host)) {
            return false;
        }
        if (isolateJBIClassLoaders != other.isolateJBIClassLoaders) {
            return false;
        }
        if (jmxRMIConnectorPort != other.jmxRMIConnectorPort) {
            return false;
        }
        if (name == null) {
            if (other.name != null) {
                return false;
            }
        } else if (!name.equals(other.name)) {
            return false;
        }
        if (password == null) {
            if (other.password != null) {
                return false;
            }
        } else if (!password.equals(other.password)) {
            return false;
        }
        if (persistenceDuration != other.persistenceDuration) {
            return false;
        }
        if (persistenceFetchSize != other.persistenceFetchSize) {
            return false;
        }
        if (recoveryCorePoolSize != other.recoveryCorePoolSize) {
            return false;
        }
        if (recoveryKeepAliveTime != other.recoveryKeepAliveTime) {
            return false;
        }
        if (persistenceDatabaseDriver == null) {
            if (other.persistenceDatabaseDriver != null) {
                return false;
            }
        } else if (!persistenceDatabaseDriver.equals(other.persistenceDatabaseDriver)) {
            return false;
        }
        if (persistenceDatabasePassword == null) {
            if (other.persistenceDatabasePassword != null) {
                return false;
            }
        } else if (!persistenceDatabasePassword.equals(other.persistenceDatabasePassword)) {
            return false;
        }
        if (persistenceDatabaseUrl == null) {
            if (other.persistenceDatabaseUrl != null) {
                return false;
            }
        } else if (!persistenceDatabaseUrl.equals(other.persistenceDatabaseUrl)) {
            return false;
        }
        if (persistenceDatabaseUser == null) {
            if (other.persistenceDatabaseUser != null) {
                return false;
            }
        } else if (!persistenceDatabaseUser.equals(other.persistenceDatabaseUser)) {
            return false;
        }
        if (repositoryDirectoryPath == null) {
            if (other.repositoryDirectoryPath != null) {
                return false;
            }
        } else if (!repositoryDirectoryPath.equals(other.repositoryDirectoryPath)) {
            return false;
        }
        if (routerQOS == null) {
            if (other.routerQOS != null) {
                return false;
            }
        } else if (!routerQOS.equals(other.routerQOS)) {
            return false;
        }
        if (routerSendAttempt != other.routerSendAttempt) {
            return false;
        }
        if (routerSendDelay != other.routerSendDelay) {
            return false;
        }
        if (routerStrategy == null) {
            if (other.routerStrategy != null) {
                return false;
            }
        } else if (!routerStrategy.equals(other.routerStrategy)) {
            return false;
        }
        if (serverPropertiesUrl == null) {
            if (other.serverPropertiesUrl != null) {
                return false;
            }
        } else if (!serverPropertiesUrl.equals(other.serverPropertiesUrl)) {
            return false;
        }
        if (sslKeyPassword == null) {
            if (other.sslKeyPassword != null) {
                return false;
            }
        } else if (!sslKeyPassword.equals(other.sslKeyPassword)) {
            return false;
        }
        if (sslKeystore == null) {
            if (other.sslKeystore != null) {
                return false;
            }
        } else if (!sslKeystore.equals(other.sslKeystore)) {
            return false;
        }
        if (sslKeystorePassword == null) {
            if (other.sslKeystorePassword != null) {
                return false;
            }
        } else if (!sslKeystorePassword.equals(other.sslKeystorePassword)) {
            return false;
        }
        if (sslTruststore == null) {
            if (other.sslTruststore != null) {
                return false;
            }
        } else if (!sslTruststore.equals(other.sslTruststore)) {
            return false;
        }
        if (sslTruststorePassword == null) {
            if (other.sslTruststorePassword != null) {
                return false;
            }
        } else if (!sslTruststorePassword.equals(other.sslTruststorePassword)) {
            return false;
        }
        if (state != other.state) {
            return false;
        }
        if (subdomainName == null) {
            if (other.subdomainName != null) {
                return false;
            }
        } else if (!subdomainName.equals(other.subdomainName)) {
            return false;
        }
        if (taskTimeout != other.taskTimeout) {
            return false;
        }
        if (tcpConnectionTimeout != other.tcpConnectionTimeout) {
            return false;
        }
        if (tcpPort != other.tcpPort) {
            return false;
        }
        if (tcpReceivers != other.tcpReceivers) {
            return false;
        }
        if (tcpSendTimeout != other.tcpSendTimeout) {
            return false;
        }
        if (tcpSenderEvictableDelay != other.tcpSenderEvictableDelay) {
            return false;
        }
        if (tcpSenderEvictorDelay != other.tcpSenderEvictorDelay) {
            return false;
        }
        if (tcpSenders != other.tcpSenders) {
            return false;
        }
        if (topologyPassPhrase == null) {
            if (other.topologyPassPhrase != null) {
                return false;
            }
        } else if (!topologyPassPhrase.equals(other.topologyPassPhrase)) {
            return false;
        }
        if (topologyDynamicLockWaitTime != other.topologyDynamicLockWaitTime) {
            return false;
        }
        if (topologyPingerStartDelay != other.topologyPingerStartDelay) {
            return false;
        }
        if (topologyPingerPeriodDelay != other.topologyPingerPeriodDelay) {
            return false;
        }
        if (transporterQueueMaxSize != other.transporterQueueMaxSize) {
            return false;
        }
        if (transporterQueueOfferingTimeout != other.transporterQueueOfferingTimeout) {
            return false;
        }
        if (user == null) {
            if (other.user != null) {
                return false;
            }
        } else if (!user.equals(other.user)) {
            return false;
        }
        if (userConfiguration == null) {
            if (other.userConfiguration != null) {
                return false;
            }
        } else if (!userConfiguration.equals(other.userConfiguration)) {
            return false;
        }
        if (workDirectoryPath == null) {
            if (other.workDirectoryPath != null) {
                return false;
            }
        } else if (!workDirectoryPath.equals(other.workDirectoryPath)) {
            return false;
        }
        return true;
    }
}
