/**
 * Copyright (c) 2005-2012 Fossil E-Commerce, 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.clientserverapi.jbi.management.installation;

import javax.jbi.JBIException;
import javax.jbi.management.InstallationServiceMBean;

import org.ow2.petals.clientserverapi.jbi.management.recovery.SystemRecoveryService;

/**
 * Client/server interface of the installation service. Few methods are added to {@link InstallationServiceMBean} for
 * reinstallation tasks of the {@link SystemRecoveryService} service.
 * 
 * @author Adrien Louis - EBM WebSourcing
 * @author Rafael Marins - Fossil E-Commerce
 */
public interface InstallationService extends javax.jbi.management.InstallationServiceMBean {

    /**
     * Uninstall a previously installed shared library with the given version.
     * 
     * @param slName
     *            the name of the shared name space to uninstall; must be
     *            non-null and non-empty
     * @param version
     *            the Version of the shared name space to uninstall; must be
     *            non-null and non-empty
     * @return true if the uninstall was successful
     */
    public boolean uninstallSharedLibrary(final String slName, final String version);

    /**
     * Try to unload installer of a component. If the component is in the
     * STARTED state it's stopped and shutdown, if the component is in the
     * STOPPED state it's shutdown. If the installer is in the INSTALLED state,
     * it's uninstalled.
     * 
     * @param compName
     *            the component unique name
     * @return true if the component installer is correctly unloaded, false
     *         otherwise
     */
    boolean forceUnloadInstaller(String compName);

    /**
     * Get the list of components that use the shared library
     * 
     * @param slName
     *            the name of the shared library
     * @param slVersion
     *            the version of the shared library
     * @return the list of components that uses this shared library
     * @throws Exception
     *             if the shared library is not installed
     */
    public String[] getInstalledComponentsForSharedLibrary(final String slName, final String slVersion)
            throws Exception;

    /**
     * Get the list of all installed shared libraries
     * 
     * @return the list of all installed shared libraries. Each shared library returned is composed of two items: name
     *         and version.
     */
    public String[][] getInstalledSharedLibraries();
    
    /**
     * Get the list of all the existing installers
     * 
     * @return the list of all the existing installers
     */
    String[] getInstallers();

    /**
     * Try to shutdown all components. If a component can't be shutdown, the
     * shutdowning process isn't aborted. It returns the list of all shutdown
     * components.
     * 
     * @return an array of succesfuly shutdown component names.
     */
    String[] shutdownAllComponents();

    /**
     * Try to shutdown all no more used components (no service assembly is
     * deployed having a service unit running on the component). If a component
     * can't be shutdown, the shutdowning process isn't aborted. It returns the
     * list of all shutdown components.
     * 
     * @return an array of succesfuly shutdown component names.
     */
    String[] shutdownAllNoMoreUsedComponents();

    /**
     * Shutdown the component. This puts the component into the
     * {@link AbstractLifeCycle#SHUTDOWN} state.
     * 
     * @param componentName
     *            name of the component to be shutdown; must be non-null and
     *            non-empty
     * @return {@code true} if the component is correctly shut down
     * @throws JBIException
     *             if the component fails to shut down
     */
    boolean shutdownComponent(String componentName) throws JBIException;

    /**
     * Try to start all components. If a component can't be started, the
     * starting process isn't aborted. It returns the list of all started
     * components.
     * 
     * @return an array of succesfuly started component names.
     */
    String[] startAllComponents();

    /**
     * Start the component. This puts the component into the
     * {@link AbstractLifeCycle#STARTED} state.
     * 
     * @param componentName
     *            name of the component to be started; must be non-null and
     *            non-empty
     * @return {@code true} if the component is correctly started
     * @throws JBIException
     *             if the component fails to start
     */
    boolean startComponent(String componentName) throws JBIException;

    /**
     * Try to stop all components. If a component can't be stoped, the stoping
     * process isn't aborted. It returns the list of all stoped components.
     * 
     * @return an array of succesfuly stoped component names.
     */
    String[] stopAllComponents();

    /**
     * Stop the component. This puts the component into the
     * {@link AbstractLifeCycle#STOPPED} state.
     * 
     * @param componentName
     *            name of the component to be stopped; must be non-null and
     *            non-empty
     * @return {@code true} if the component is correctly stopped
     * @throws JBIException
     *             if the component fails to stop
     */
    boolean stopComponent(String componentName) throws JBIException;
    
    /**
     * Try to uninstall all components. If a component can't be uninstalled, the
     * uninstalling process isn't aborted. It returns the list of all
     * uninstalled components.
     * 
     * @return an array of succesfuly uninstalled component names.
     */
    String[] uninstallAllComponents();

    /**
     * Try to uninstall all no more used components (no service assembly is
     * deployed having a service unit running on the component). If a component
     * can't be uninstalled, the uninstalling process isn't aborted. It returns
     * the list of all uninstalled components.
     * 
     * @return an array of succesfuly uninstalled component names.
     */
    String[] uninstallAllNoMoreUsedComponents();

    /**
     * Try to uninstall all shared libraries. If a shared library can't be
     * uninstalled, the uninstalling process isn't aborted. It returns the list
     * of all uninstalled shared libraries.
     * 
     * @return an array of succesfuly uninstalled shared library names.
     */
    String[] uninstallAllSharedLibraries();

    /**
     * Try to uninstall all no more used shared libraries (no component is
     * installed referencing the shared library). If a shared library can't be
     * uninstalled, the uninstalling process isn't aborted. It returns the list
     * of all uninstalled shared libraries.
     * 
     * @return an array of succesfuly uninstalled shared library names.
     */
    String[] uninstallAllNoMoreUsedSharedLibraries();

    /**
     * Uninstall the component. This puts the component into the
     * {@link Installer#UNINSTALLED} state.
     * 
     * @param componentName
     *            name of the component to be uninstalled; must be non-null and
     *            non-empty
     * @return {@code true} if the component is correctly uninstalled
     * @throws JBIException
     *             if the component fails to uninstall
     */
    boolean uninstallComponent(String componentName) throws JBIException;

    /**
     * Try to uninstall all components. If the boolean is set to true, the
     * uninstallation process is forced as this : if the component is in the
     * STARTED state it's stopped and shutdown, if the component is in the
     * STOPPED state it's shutdown. If the installer is in the INSTALLED state,
     * it's uninstalled. If a component can't be uninstalled, the uninstalling
     * process isn't aborted. It returns the list of all uninstalled components.
     * 
     * @return an array of succesfuly uninstalled component names.
     */
    String[] unloadAllInstallers(boolean force);

}
