/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.clientserverapi.topology;

import java.util.Map;
import java.util.Set;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.topology.exception.AddContainerException;
import org.ow2.petals.clientserverapi.topology.exception.AttachContainerException;
import org.ow2.petals.clientserverapi.topology.exception.DetachContainerException;
import org.ow2.petals.clientserverapi.topology.exception.InconsistencyException;
import org.ow2.petals.clientserverapi.topology.exception.InconsistentSubdomainNameException;
import org.ow2.petals.clientserverapi.topology.exception.NoContainerConfigurationProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoContainerNameProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoSecurityPassPhraseProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoSubdomainConfigurationProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.SubDomainNotFoundException;
import org.ow2.petals.clientserverapi.topology.exception.SubdomainIsNotDynamicException;
import org.ow2.petals.clientserverapi.topology.exception.TopologyServiceException;


/**
 * Client interface of the service 'Topology'
 * 
 * @author Christophe DENEUX - Linagora
 */
public interface TopologyService {

    /**
     * Default value of the dynamic topology lock wait time, in seconds
     */
    public final static int DEFAULT_DYNAMIC_TOPOLOGY_LOCK_WAIT_TIME = 30;

    /**
     * Default value of the start delay of the topology pinger, in milliseconds
     */
    public final static long DEFAULT_TOPOLOGY_PINGER_START_DELAY = 2000;

    /**
     * Default value of the period delay of the topology pinger, in milliseconds
     */
    public final static long DEFAULT_TOPOLOGY_PINGER_PERIOD_DELAY = 5000;

    /**
     * Name and values of the property identifying a configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class TopologyItem {

        /**
         * The property name
         */
        public static final String TOPOLOGY_TYPE = "type";

        /**
         * Value indentifying a domain configuration
         */
        public static final String DOMAIN = "domain";

        /**
         * Value indentifying a sub-domain configuration
         */
        public static final String SUBDOMAIN = "subdomain";

        /**
         * Value indentifying a sub-domain configuration
         */
        public static final String CONTAINER = "container";
    }

    /**
     * Names of properties contained into a domain configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class DomainPropertyName {

        /**
         * Domain key name: domain name
         */
        public static final String CONF_DOMAIN_NAME = "domainName";

        /**
         * Domain key name: domain Description
         */
        public static final String CONF_DOMAIN_DESCRIPTION = "domainDescription";

        /**
         * Domain key name: JNDI Factory
         */
        public static final String CONF_JNDI_FACTORY = "jndiFactory";

        /**
         * Domain key name: JNDI Provider URL
         */
        public static final String CONF_JNDI_PROVIDER_URL = "jndiProviderUrl";

        /**
         * Domain key name: JNDI Security Principal
         */
        public static final String CONF_JNDI_SECURITY_PRINCIPAL = "jndiSecurityPrincipal";

        /**
         * Domain key name: JNDI Security Credentials
         */
        public static final String CONF_JNDI_SECURITY_CREDENTIALS = "jndiSecurityCredentials";

        /**
         * Domain key name: JNDI Pool Size
         */
        public static final String CONF_JNDI_POOL_SIZE = "jndiPoolSize";

        /**
         * Domain key name: JNDI Pool Size
         */
        public static final String CONF_JNDI_BATCH_SIZE = "jndiBatchSize";
    }

    /**
     * Names of properties contained into a sub-domain configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class SubdomainPropertyName {

        /**
         * Subdomain key name: Subdomain Name
         */
        public static final String CONF_SUBDOMAIN_NAME = "subdomainName";

        /**
         * Subdomain key name: Subdomain Description
         */
        public static final String CONF_SUBDOMAIN_DESCRIPTION = "subdomainDescription";

        /**
         * Subdomain key name: Subdomain mode
         */
        public static final String CONF_SUBDOMAIN_MODE = "subdomainMode";

        /**
         * Subdomain key name: Subdomain Name
         */
        public static final String CONF_NETWORK_TIME_SYNCHRONIZED = "networkTimeSynchronized";

        /**
         * Subdomain key name: Registry implementation
         */
        public static final String CONF_REGISTRY_IMPLEMENTATION = "registryImplementation";
    }

    /**
     * Values of properties contained into the sub-domain configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class SubdomainPropertyValues {

        /**
         * Subdomain mode: 'static'
         */
        public static final String MODE_STATIC = "static";

        /**
         * Subdomain mode: 'dynamic'
         */
        public static final String MODE_DYNAMIC = "dynamic";

    }

    /**
     * Names of properties contained into a container configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class ContainerPropertyName {

        /**
         * Container key name: Container Name
         */
        public static final String CONF_CONTAINER_NAME = "containerName";

        /**
         * Container key name: Container Description
         */
        public static final String CONF_CONTAINER_DESCRIPTION = "containerDescription";

        /**
         * Container key name: Sub-domain Name
         */
        public static final String CONF_SUBDOMAIN_NAME = "subdomainName";

        /**
         * Container key name: Container Host Name
         */
        public static final String CONF_HOST = "host";

        /**
         * Container key name: User
         */
        public static final String CONF_USER = "user";

        /**
         * Container key name: Password
         */
        public static final String CONF_PWD = "password";

        /**
         * Container key name: JMX RMI server port
         */
        public static final String CONF_JMX_RMI_PORT = "jmxRMIPort";

        /**
         * Container key name: transport TCP port
         */
        public static final String CONF_TRANSPORT_TCP_PORT = "transportTCPPort";

        /**
         * Container key name: Container state
         */
        public static final String CONF_STATE = "state";
    }

    /**
     * Values of properties contained into the container configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class ContainerPropertyValues {
        /**
         * Container State: Started
         */
        public static final String STATE_STARTED = "STARTED";

        /**
         * Container State: Stopped
         */
        public static final String STATE_STOPPED = "STOPPED";

    }

    /**
     * Default values of properties contained into the container configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class ContainerPropertyDefaultValues {
        /**
         * Default JMX RMI server port
         */
        public static final String CONF_JMX_RMI_PORT = "";

        /**
         * Default transport TCP port
         */
        public static final String CONF_TRANSPORT_TCP_PORT = "";

    }

    /**
     * <p>
     * Add the given container into the given sub-domain of the topology. The add is done validating the consistency of
     * the resulting topology.
     * </p>
     * <p>
     * <s>Note:</s> the sub-domain is defined inside the container itself. See
     * {@link ContainerConfiguration#setSubdomainName(String)}
     * 
     * @param addedContainer
     *            the container to add to the current topology. Can not be <code>null</code>. Its sub-domain field can
     *            not be <code>null</code>
     * 
     * @throws NoContainerConfigurationProvidedException
     *             The container to add is <code>null</code>
     * @throws NoSubdomainConfigurationProvidedException
     *             The sub-domain name is not defined inside the container configuration (<code>addedContainer</code>).
     * @throws SubDomainNotFoundException
     *             The sub-domain in which the container must be added does not exist
     * @throws SubdomainIsNotDynamicException
     *             The provided sub-domain is not <b>dynamic</b>. Adding a container in such sub-domain is prohibited.
     * @throws AddContainerException
     *             Failed to add the container to the current topology
     */
    void addContainerConfiguration(final ContainerConfiguration addedContainer)
            throws NoContainerConfigurationProvidedException, NoSubdomainConfigurationProvidedException,
            SubDomainNotFoundException, SubdomainIsNotDynamicException, AddContainerException;

    /**
     * Retrieve the topology currently used, as collections of containers, sub-domains and domain configurations.
     * 
     * @param securityPassPhrase
     *            Passphrase needed to return sensible information. If <code>null</code>, empty or not matching the one
     *            of the container on which the connection is established, no sensible information will be returned.
     * @return a {@link Set} of containers, sub-domains and domain configurations in key/value form. An empty property
     *         is associated to an unset parameter.
     * @throws TopologyServiceException
     *             An error occurs
     */
    public Set<Map<String, String>> retrieveTopology(final String securityPassPhrase) throws PetalsException;

    /**
     * <p>
     * Attach the current container to the sub-domain of the given container.
     * </p>
     * <p>
     * Attaching the current container consists of:
     * <ol>
     * <li>get the registry configuration of the sub-domain in which the container will be attached,</li>
     * <li>stopping router and transporters of the container to attach,</li>
     * <li>deregister from the initial topology all endpoints of the container to attach,</li>
     * <li>restart the registry to use the registry of the new sub-domain,</li>
     * <li>register all endpoints of the container to attach on the new registry,</li>
     * <li>start router and transporters of the container to attach</li>
     * </ol>
     * </p>
     * <p>
     * </p>
     * The current container interacts with the container of its next sub-domain using JMX. JMX connection information
     * are defined in the given container configuration.
     * <p>
     * If the given container is not running, not available or unreachable, the attachment is aborted.
     * </p>
     * <p>
     * About the given container attributes:
     * <ul>
     * <li>if no host is set, <code>localhost</code> is used as hostname of the given container</li>
     * <li>if no port is set, {@link JMXClient.DEFAULT_PORT} is used for the given container</li>
     * <li>the attribute 'sub-domain' is required, otherwise {@linkNoSubdomainConfigurationProvidedException} is raised</li>
     * <li>the JMX credentials are required if the given container is secured</li>
     * </ul>
     * </p>
     * 
     * @param container
     *            A running container of the sub-domain in which the current container will be added. Not
     *            <code>null</code>.
     * @param securityPassPhrase
     *            The security pass-phrase of the given container is required to attach a container. Not
     *            <code>null</code>.
     * 
     * @throws NoContainerConfigurationProvidedException
     *             The container to add is <code>null</code>
     * @throws NoContainerNameProvidedException
     *             The container name is not defined inside the container configuration (<code>container</code>).
     * @throws NoSubdomainConfigurationProvidedException
     *             The sub-domain name is not defined inside the container configuration (<code>container</code>).
     * @throws SubdomainIsNotDynamicException
     *             The current or target sub-domain is not <b>dynamic</b>. Using a container in such sub-domain is
     *             prohibited.
     * @throws InconsistentNameException
     *             The container name set in the given container configuration is inconsistent with the topology of the
     *             remote container (it does not exist in the target topology)
     * @throws InconsistentSubdomainNameException
     *             The sub-domain set in the given container configuration is inconsistent with the topology of the
     *             remote container
     * @throws NoSecurityPassPhraseProvidedException
     *             No security pass-phrase provided, or <code>null</code>
     * @throws AttachContainerException
     *             Failed to attach the container to the sub-domain of the given container.
     */
    public void attachContainerTo(final ContainerConfiguration container, final String securityPassPhrase)
            throws NoContainerConfigurationProvidedException, NoContainerNameProvidedException,
            NoSubdomainConfigurationProvidedException, SubdomainIsNotDynamicException, InconsistencyException,
            NoSecurityPassPhraseProvidedException, AttachContainerException;

    /**
     * <p>
     * Detach the current container from its sub-domain.
     * </p>
     * <p>
     * Detaching the current container consists of:
     * <ol>
     * <li>stopping router and transporters of the container to detach,</li>
     * <li>deregister all endpoints of the container to detach,</li>
     * <li>remove the current container from the topology,</li>
     * <li>set a dummy topology to the container to detach.</li>
     * </ol>
     * </p>
     * <p>
     * 
     * @throws SubdomainIsNotDynamicException
     *             The current sub-domain is not <b>dynamic</b>.
     * @throws DetachContainerException
     *             Failed to attach the container to the sub-domain of the given container.
     */
    public void detachContainer() throws SubdomainIsNotDynamicException, DetachContainerException;

    /**
     * Retrieve the registry implementation of the sub-domain of the current container
     * 
     * @return The registry implementation of the sub-domain. Not null.
     * @throws PetalsException
     *             An error occurs
     */
    public String retrieveRegistryImplementation() throws PetalsException;

}
