/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.clientserverapi.configuration;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration.ContainerState;
import org.ow2.petals.clientserverapi.topology.TopologyService.ContainerPropertyDefaultValues;
import org.ow2.petals.clientserverapi.topology.TopologyService.ContainerPropertyName;
import org.ow2.petals.clientserverapi.topology.TopologyService.ContainerPropertyValues;

/**
 * Unit tests of {@link ContainerConfiguration}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ContainerConfigurationTest {

    private final static String CONTAINER_NAME = "container-name-test";

    private final static String CONTAINER_DESCRIPTION = "container description test";

    private final static String CONTAINER_SUBDOMAIN_NAME = "container-subdomain-name-test";

    private final static String CONTAINER_HOST = "container-host-test";

    private final static String CONTAINER_USER = "container-user-test";

    private final static String CONTAINER_PWD = "container-pwd-test";

    private ContainerConfiguration containerConfig;

    @Before
    public void before() {
        this.containerConfig = new ContainerConfiguration();
        this.containerConfig.setName(CONTAINER_NAME);
        this.containerConfig.setDescription(CONTAINER_DESCRIPTION);
        this.containerConfig.setSubdomainName(CONTAINER_SUBDOMAIN_NAME);
        this.containerConfig.setHost(CONTAINER_HOST);
        this.containerConfig.setUser(CONTAINER_USER);
        this.containerConfig.setPassword(CONTAINER_PWD);
        this.containerConfig.setJmxRMIConnectorPort(0);
        this.containerConfig.setTCPPort(0);
        this.containerConfig.setState(ContainerState.STOPPED);
    }

    /**
     * <p>
     * Check the conversion of a {@link ContainerConfiguration} into a {@link Map}, and the {@link Map} into a
     * {@link ContainerConfiguration}, where:
     * <ul>
     * <li>all fields put in map are set with a value,</li>
     * <li>the conversion to map is done with security on.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testToMap_WithSecurity() {
        final Map<String, String> containerConfigMap = this.containerConfig.toMap(true);

        this.assertBaseMap(containerConfigMap, CONTAINER_NAME, CONTAINER_DESCRIPTION, CONTAINER_SUBDOMAIN_NAME,
                CONTAINER_HOST, ContainerPropertyDefaultValues.CONF_JMX_RMI_PORT,
                ContainerPropertyDefaultValues.CONF_TRANSPORT_TCP_PORT, ContainerPropertyValues.STATE_STOPPED);

        assertEquals("Unexpected container user", CONTAINER_USER,
                containerConfigMap.get(ContainerPropertyName.CONF_USER));
        assertEquals("Unexpected container password", CONTAINER_PWD,
                containerConfigMap.get(ContainerPropertyName.CONF_PWD));

        final ContainerConfiguration reversedContainerConfig = new ContainerConfiguration(containerConfigMap);
        assertEquals("Reversed container configuration does not match the original container configuration",
                this.containerConfig, reversedContainerConfig);
    }

    /**
     * <p>
     * Check the conversion of a {@link ContainerConfiguration} into a {@link Map}, and the {@link Map} into a
     * {@link ContainerConfiguration}, where:
     * <ul>
     * <li>all fields put in map are set with a value,</li>
     * <li>except the description</li>
     * <li>the conversion to map is done with security on.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testToMap_WithoutDescription() {
        this.containerConfig.setDescription(null);
        final Map<String, String> containerConfigMap = this.containerConfig.toMap(true);

        this.assertBaseMap(containerConfigMap, CONTAINER_NAME, null, CONTAINER_SUBDOMAIN_NAME, CONTAINER_HOST,
                ContainerPropertyDefaultValues.CONF_JMX_RMI_PORT,
                ContainerPropertyDefaultValues.CONF_TRANSPORT_TCP_PORT, ContainerPropertyValues.STATE_STOPPED);

        final ContainerConfiguration reversedContainerConfig = new ContainerConfiguration(containerConfigMap);
        assertEquals("Reversed container configuration does not match the original container configuration",
                this.containerConfig, reversedContainerConfig);
    }

    /**
     * <p>
     * Check the conversion of a {@link ContainerConfiguration} into a {@link Map}, and the {@link Map} into a
     * {@link ContainerConfiguration}, where:
     * <ul>
     * <li>all fields put in map are set with a value,</li>
     * <li>the conversion to map is done with security <b>off</b>.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testToMap_WithoutSecurity() {
        final Map<String, String> containerConfigMap = this.containerConfig.toMap(false);

        this.assertBaseMap(containerConfigMap, CONTAINER_NAME, CONTAINER_DESCRIPTION, CONTAINER_SUBDOMAIN_NAME,
                CONTAINER_HOST, ContainerPropertyDefaultValues.CONF_JMX_RMI_PORT,
                ContainerPropertyDefaultValues.CONF_TRANSPORT_TCP_PORT, ContainerPropertyValues.STATE_STOPPED);

        assertNull("Container user returned", containerConfigMap.get(ContainerPropertyName.CONF_USER));
        assertNull("Container password returned", containerConfigMap.get(ContainerPropertyName.CONF_PWD));

        // Update the original container configuration to be compliant with no security
        this.containerConfig.setUser(null);
        this.containerConfig.setPassword(null);
        final ContainerConfiguration reversedContainerConfig = new ContainerConfiguration(containerConfigMap);
        assertEquals("Reversed container configuration does not match the original container configuration",
                this.containerConfig, reversedContainerConfig);
    }

    /**
     * <p>
     * Check the conversion of a {@link ContainerConfiguration} into a {@link Map}, and the {@link Map} into a
     * {@link ContainerConfiguration}, where:
     * <ul>
     * <li>all fields put in map are set with a value,</li>
     * <li>the JMX port is set explicitly to a value different of the default value,</li>
     * <li>the conversion to map is done with security on.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testToMap_WithDedicatedJmxPort() {

        final String expectedJmxPort = "12345";
        this.containerConfig.setJmxRMIConnectorPort(Integer.parseInt(expectedJmxPort));
        final Map<String, String> containerConfigMap = this.containerConfig.toMap(true);

        this.assertBaseMap(containerConfigMap, CONTAINER_NAME, CONTAINER_DESCRIPTION, CONTAINER_SUBDOMAIN_NAME,
                CONTAINER_HOST, expectedJmxPort, ContainerPropertyDefaultValues.CONF_TRANSPORT_TCP_PORT,
                ContainerPropertyValues.STATE_STOPPED);

        final ContainerConfiguration reversedContainerConfig = new ContainerConfiguration(containerConfigMap);
        assertEquals("Reversed container configuration does not match the original container configuration",
                this.containerConfig, reversedContainerConfig);
    }

    /**
     * <p>
     * Check the conversion of a {@link ContainerConfiguration} into a {@link Map}, and the {@link Map} into a
     * {@link ContainerConfiguration}, where:
     * <ul>
     * <li>all fields put in map are set with a value,</li>
     * <li>the transport TCP port is set explicitly to a value different of the default value,</li>
     * <li>the conversion to map is done with security on.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testToMap_WithDedicatedTransportTcpPort() {

        final String expectedTcpPort = "12345";
        this.containerConfig.setTCPPort(Integer.parseInt(expectedTcpPort));
        final Map<String, String> containerConfigMap = this.containerConfig.toMap(true);

        this.assertBaseMap(containerConfigMap, CONTAINER_NAME, CONTAINER_DESCRIPTION, CONTAINER_SUBDOMAIN_NAME,
                CONTAINER_HOST, ContainerPropertyDefaultValues.CONF_JMX_RMI_PORT, expectedTcpPort,
                ContainerPropertyValues.STATE_STOPPED);

        final ContainerConfiguration reversedContainerConfig = new ContainerConfiguration(containerConfigMap);
        assertEquals("Reversed container configuration does not match the original container configuration",
                this.containerConfig, reversedContainerConfig);
    }

    /**
     * <p>
     * Check the conversion of a {@link ContainerConfiguration} into a {@link Map}, and the {@link Map} into a
     * {@link ContainerConfiguration}, where:
     * <ul>
     * <li>all fields put in map are set with a value,</li>
     * <li>the container state is set explicitly to a value different of the default value,</li>
     * <li>the conversion to map is done with security on.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testToMap_WithStartedContainer() {

        this.containerConfig.setState(ContainerState.STARTED);
        final Map<String, String> containerConfigMap = this.containerConfig.toMap(true);

        this.assertBaseMap(containerConfigMap, CONTAINER_NAME, CONTAINER_DESCRIPTION, CONTAINER_SUBDOMAIN_NAME,
                CONTAINER_HOST, ContainerPropertyDefaultValues.CONF_JMX_RMI_PORT,
                ContainerPropertyDefaultValues.CONF_TRANSPORT_TCP_PORT, ContainerPropertyValues.STATE_STARTED);

        final ContainerConfiguration reversedContainerConfig = new ContainerConfiguration(containerConfigMap);
        assertEquals("Reversed container configuration does not match the original container configuration",
                this.containerConfig, reversedContainerConfig);
    }

    private void assertBaseMap(final Map<String, String> containerConfigMap, final String containerName,
            final String containerDescr, final String containerSubdomain, final String containerHost,
            final String containerJmxPort, final String containerTransportTcpPort, final String containerState) {
        assertEquals("Unexpected container name", containerName,
                containerConfigMap.get(ContainerPropertyName.CONF_CONTAINER_NAME));
        if (containerDescr != null) {
            assertEquals("Unexpected container description", containerDescr,
                    containerConfigMap.get(ContainerPropertyName.CONF_CONTAINER_DESCRIPTION));
        } else {
            assertNull("Container description returned",
                    containerConfigMap.get(ContainerPropertyName.CONF_CONTAINER_DESCRIPTION));
        }
        assertEquals("Unexpected container sub-domain", containerSubdomain,
                containerConfigMap.get(ContainerPropertyName.CONF_SUBDOMAIN_NAME));
        assertEquals("Unexpected container host", containerHost,
                containerConfigMap.get(ContainerPropertyName.CONF_HOST));
        assertEquals("Unexpected container jmx rmi port", containerJmxPort,
                containerConfigMap.get(ContainerPropertyName.CONF_JMX_RMI_PORT));
        assertEquals("Unexpected container transport tcp port", containerTransportTcpPort,
                containerConfigMap.get(ContainerPropertyName.CONF_TRANSPORT_TCP_PORT));
        assertEquals("Unexpected container state", containerState,
                containerConfigMap.get(ContainerPropertyName.CONF_STATE));
    }

}
